#!/usr/bin/perl

###############################################################
# Link Admin v 1.1 Link Admin Interface Script                #
###############################################################
# index.cgi v 1.1 RJ 04-Mar-2002                              #
# This processes standard admin requests and passes them to   #
# the relevant module.                                        #
#                                                             #
# For further information please visit:                       #
# http://www.UsefulScripts.com/                               #
###############################################################
# Copyright  2001 iNet Publishing Ltd                        #
# Reproduction in part or in whole is strictly prohibited.    #
# This copyright header must not be removed.                  #
# No warranty expressed or implied.                           #
###############################################################


# Version 1.2 26-Jan-2004
# Added additional code for new version of Perl handling DBM files
# Version 1.1 04-Mar-2002
# Added additional safety guards relating to Text format data and writable files
# Version 1.0 13-Nov-2001
# Original script


###############################################################
#           DO NOT EDIT ANYTHING BENEATH THIS LINE!           #
###############################################################

use DB_File;

# Load in the configuration file.
dbmopen (%CONF,"./config/core.Config",0777);
$DATA{'core.Version'} = $CONF{'core.Version'} = "1.2";

# Load in the core library containing shared functions
require ($CONF{'scriptWorkingDir'} . "library/core.Library.pl");

&parseFormInput;

if ($ENV{'REMOTE_USER'}) { $in{'thisReviewer'} = $ENV{'REMOTE_USER'}; }
else { $in{'thisReviewer'} = "Default"; }

if (!$processModule) { print "Location: http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}/Main/view\n\n"; exit; }

# If there is authorised restrictions on who can view what activated
if ($CONF{'enableAuthRestrictions'}) {
	# Load in the authorisation file.
	dbmopen (%AUTH,"./config/core.Auth",0777);
	unless ($AUTH{$processModule} =~ /:$in{'thisReviewer'}:/) {
		&printErrorMessage($englishErrorMessages{"unauthorisedErrorPrefix"},"<li>".$englishErrorMessages{"unauthorisedAccess"}."\n");
		exit;
	}
}

if ($processModule eq "Main") { &processMain; }
elsif ($processModule eq "Review") { &processReview; }
elsif ($processModule eq "Search") { &processSearch; }
elsif ($processModule eq "Detail") { &processDetail; }
elsif ($processModule eq "Modify") { &processModify; }
elsif ($processModule eq "MultiReject") { &processMultiReject; }
elsif ($processModule eq "Add") { &processAdd; }
elsif ($processModule eq "Refresh") { &processRefresh; }
elsif ($processModule eq "Blocked") { &processBlocked; }
elsif ($processModule eq "Categories") { &processCategories; }
elsif ($processModule eq "Templates") { &processTemplates; }
elsif ($processModule eq "Reasons") { &processReasons; }
elsif ($processModule eq "Configure") { &processConfigure; }
elsif ($processModule eq "Formhtml") { &processFormhtml; }
elsif (-e $CONF{'scriptWorkingDir'} . "library/module.$processModule.pl") { require ( $CONF{'scriptWorkingDir'} . "library/module.$processModule.pl"); &runModule; }
else { print "Location: http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}/Main/view\n\n"; exit; }

exit;


###############################################################
#                    MAIN PAGE SUB ROUTINE                    #
###############################################################
sub processMain {

	open (REFRESHTIME,$CONF{'scriptWorkingDir'} . "data/data.LastRefresh.txt"); 
	$lastRefresh = (<REFRESHTIME>);
	close (REFRESHTIME);
	$INFO{'lastRefresh'} = &formatTimestamp($lastRefresh,"datetime");

	require ($CONF{'scriptWorkingDir'} . "library/module." . $CONF{'dataStorage'} . ".pl");
	($INFO{'submittedLinks'},$INFO{'friendsLinks'}) = &getSubmittedLinks;
	($INFO{'totalLinks'},$INFO{'listedLinks'},$INFO{'reviewedLinks'}) = &getReviewedLinks;
	if ($CONF{'multiReviewers'}) { 
		require ($CONF{'scriptWorkingDir'} . "library/module.MultiReviewer.pl");
		($reviewableSites,$reviewableFriendsSites) = &getReviewableLinks; 
		$INFO{'reviewableSites'} = "$reviewableSites / "; 
		$INFO{'reviewableFriendsSites'} = "$reviewableFriendsSites / ";
		$INFO{'multiReviewers'} = 	"<p><a href=\"../MultiReviewer/view\"><b>Reviewer Logs/Reviewing Now</b></a>\n".
									"<small><br>See who is reviewing right now and a summary of total reviews</small></p>\n";

	}

	&displayPage("AdminMain","");
}




###############################################################
#                     REVIEW SUB ROUTINE                      #
###############################################################
# Display the configured number of submissions for review
sub processReview {
	# Load in the module for the specific type of data storage being used
	require ($CONF{'scriptWorkingDir'} . "library/module." . $CONF{'dataStorage'} . ".pl");
	if ($CONF{'multiReviewers'}) { 
		require ($CONF{'scriptWorkingDir'} . "library/module.MultiReviewer.pl"); 
	}

	# Check that the data files are writable if the data format is Text
	if ($CONF{'dataStorage'} eq "Text") {
		$errorResponse = "";
		# We need to step through each page in case they do not exist
		if (-e "$CONF{'scriptWorkingDir'}data/data.AcceptedLinks.txt") {	
			unless (-w "$CONF{'scriptWorkingDir'}data/data.AcceptedLinks.txt") { $errorResponse = "<li>$CONF{'scriptWorkingDir'}data/data.AcceptedLinks.txt</li>\n"; }
		}
		if (-e "$CONF{'scriptWorkingDir'}data/data.RejectedLinks.txt") { 
			unless (-w "$CONF{'scriptWorkingDir'}data/data.RejectedLinks.txt") { $errorResponse .= "<li>$CONF{'scriptWorkingDir'}data/data.RejectedLinks.txt</li>\n"; }
		}
		if (-e "$CONF{'scriptWorkingDir'}data/data.RejectedLinks.txt") { 
			unless (-w "$CONF{'scriptWorkingDir'}data/data.RejectedLinks.txt") { $errorResponse .= "<li>$CONF{'scriptWorkingDir'}data/data.RejectedLinks.txt</li>\n"; }
		}
		if (-e "$CONF{'scriptWorkingDir'}data/data.NewSubmissions.txt") { 
			unless (-w "$CONF{'scriptWorkingDir'}data/data.NewSubmissions.txt") { $errorResponse .= "<li>$CONF{'scriptWorkingDir'}data/data.NewSubmissions.txt</li>\n"; }
		}
		if (-e "$CONF{'scriptWorkingDir'}data/data.AllSubmissions.txt") { 
			unless (-w "$CONF{'scriptWorkingDir'}data/data.AllSubmissions.txt") { $errorResponse .= "<li>$CONF{'scriptWorkingDir'}data/data.AllSubmissions.txt</li>\n"; }
		}

		if ($errorResponse) { &printErrorMessage("Reviewing is not possible because the following files are not writable:",$errorResponse); exit; }			
	}				

	if ($processAction eq "view") {
	
		# Get the number of submissions
		($INFO{'submittedLinks'},$INFO{'friendsLinks'}) = &getSubmittedLinks;
		if ($in{'type'} eq "Friends") { $availableLinks = $INFO{'friendsLinks'}; }
		else { $availableLinks = $INFO{'submittedLinks'}; }
		# Get the submissions to be shown
		&getSubmissions;
		if ($in{'specificSubmission'}) { $DATA{'specificSubmission'} = $in{'specificSubmission'}; }
		$INFO{'shownLinks'} = scalar(@linksToShow);

		if ($CONF{'multiReviewers'}) { 
			$INFO{'bodyOnload'} = " onLoad=\"JavaScript: countDown(900)\"";
		}
		open (REVIEWTEMPLATE,$CONF{'scriptWorkingDir'} . "templates/template.LinkReview.txt");
		while (<REVIEWTEMPLATE>) { $origReviewFormat .= $_; }
		close (REVIEWTEMPLATE);

		print "Content-type: text/html\n\n";
		&printTemplate($CONF{'scriptWorkingDir'} . "templates/template.AdminHeader.txt");
		if ($CONF{'multiReviewers'}) { 
			print 	"<script language=\"JavaScript\">\n".
					"function countDown(tick) {\n".
					"if (tick == 0) {\n".
					"window.location.href=\"$ENV{'REQUEST_URI'}\";\n".
					"return;\n".
					"}\n\n".
					"--tick;\n".
					"var command = \"countDown(\" + tick + \")\";\n".
					"window.setTimeout(command,1000);\n".
					"}\n".
					"</script>\n\n";
		}


		print 	"<table width=600 cellpadding=0 cellspacing=0 border=0 align=center><tr><td colspan=2><p><font size=2>".
				"<a href=\"javaScript: helpWindow('http://www.usefulscripts.com/showHelp.phtml?script=LinkAdmin1_0&page=Review.phtml')\" onMouseOver=\"status='Show Help For This Section'; return true;\" onMouseOut=\"status='';\"\">".
				"<img src=\"$CONF{'laImageDir'}questionWhite.gif\" align=right height=17 width=17 border=0 alt=\"Show Help For This Section\"></a>".
				"<b>$in{'type'} Link Review:</b><br>".
				"<small>Showing <b>$INFO{'shownLinks'}</b> of <b>";
		# If the Multiple Reviewer module is being used then get the submissions available to this reveiwer
		if ($CONF{'multiReviewers'}) { 
			($reviewableSites,$reviewableFriendsSites) = &getReviewableLinks; 
			$INFO{'reviewableSites'} = "$reviewableSites / "; 
			$INFO{'reviewableFriendsSites'} = "$reviewableFriendsSites / ";			
			if ($in{'type'} eq "General") { print $INFO{'reviewableSites'}; }
			elsif ($in{'type'} eq "Friends") { print $INFO{'reviewableFriendsSites'}; }
		}
		print	"$availableLinks</b> Submissions ($in{'type'})</small><br>".
				"Review the links below and accept/reject them or leave them pending. Press the submit button at the bottom ".
				"to save any changes.</p></td></tr>".
				"</table>".
				"<table border=0 width=600 cellpadding=0 cellspacing=0 align=center><tr><td height=3></td></tr>".
				"<tr><td bgcolor=#2255a0 height=1>".
				"<table border=0 width=600 cellpadding=0 cellspacing=0><tr><td></td></tr></table>".
				"</td></tr><tr><td height=10><form action=save method=POST></td></tr></table>\n";

		if (-e $CONF{'scriptWorkingDir'} . "data/data.SubCats.txt" && (stat($CONF{'scriptWorkingDir'} . "data/data.SubCats.txt")) [7] > 0) { 
			open (SUBCATS,$CONF{'scriptWorkingDir'} . "data/data.SubCats.txt"); 
			while (<SUBCATS>) { chop($_); push (@subCategories,$_); }
			close (SUBCATS);
		}
		open (CATS,$CONF{'scriptWorkingDir'} . "data/data.Categories.txt"); 
		@categories = (<CATS>);
		close (CATS);
		open (REJECTS,$CONF{'scriptWorkingDir'} . "data/data.Reasons.txt"); 
		@rejectReasons = (<REJECTS>);
		close (REJECTS);

		foreach $reason (@rejectReasons) { chop($reason); $INFO{'rejectReason'} .= "<option>$reason</option>"; }

		# Increment the field numbers
		$INFO{'reviewNumber'} = 1;

		foreach $linkToShow (@linksToShow) {
			$reviewFormat = $origReviewFormat;
			@linkData = split(/\t/,$linkToShow);
			$y=0; foreach $fieldName (@linksFieldOrder) { $DATA{$fieldName} = @linkData[$y]; $y++; }

			if ($DATA{'recipURL'}) { $INFO{'recipURL'} = "<b>Recip Link URL:</b> <a href=\"$DATA{'recipURL'}\" target=\"recipNew\">$DATA{'recipURL'}</a> ($DATA{'recipStatus'})<br>\n"; }
			else { undef($INFO{'recipURL'}); }
			$INFO{'properTimeSubmitted'} = &formatTimestamp($DATA{'timeSubmitted'},"datetime");
			if (@subCategories) {
				$INFO{'siteSubCategory'} = "<select name=linkSubCategory$INFO{'reviewNumber'}>";
				foreach $subCat (@subCategories) { 
					$INFO{'siteSubCategory'} .= "<option";
					if ($subCat eq $DATA{'siteSubCategory'}) { $INFO{'siteSubCategory'} .= " selected"; }
					$INFO{'siteSubCategory'} .= ">$subCat</option>";
				}
				$INFO{'siteSubCategory'} .= "</select>";
			}

			$INFO{'siteCategory'} = &getCategoryList($DATA{'siteCategory'});

			while ($reviewFormat =~ /%%([a-z]+):([a-z]+):([a-z]*):([a-z0-9]*)%%/ig) {
				undef $toReplaceWith;
				$insertDataType = $1;
				$insertDataKey = $2;
				$insertFieldType = $3;
				$insertFieldValue = $4;
				if ($insertDataType =~ /^CONF/) { 
					if ($insertFieldType eq "checkbox") { if ($CONF{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "checked"; } }
					else { $toReplaceWith = $CONF{$insertDataKey}; }
				}
				elsif ($insertDataType =~ /^INFO/) { 
					if ($insertFieldType eq "checkbox") { if ($INFO{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "checked"; } }
					else { $toReplaceWith = $INFO{$insertDataKey}; }
				}
				elsif ($insertDataType =~ /^DATA/) { 
					if ($insertFieldType eq "checkbox") { if ($DATA{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "checked"; } }
					else { $toReplaceWith = $DATA{$insertDataKey}; }
				}
				$reviewFormat =~ s/%%$insertDataType:$insertDataKey:$insertFieldType:$insertFieldValue%%/$toReplaceWith/g;
			}
			print $reviewFormat;
			$INFO{'reviewNumber'}++;
			if ($INFO{'reviewNumber'} <= scalar(@linksToShow)) { 
				print 	"<table border=0 width=600 cellpadding=0 cellspacing=0 align=center><tr><td height=10></td></tr>".
						"<tr><td bgcolor=#2255a0 height=1>".
						"<table border=0 width=600 cellpadding=0 cellspacing=0><tr><td></td></tr></table>".
						"</td></tr><tr><td height=10></td></tr></table>\n";
			 }
		}


		if ($availableLinks && $INFO{'shownLinks'}) {
			print 	"<input type=hidden name=shownLinks value=$INFO{'shownLinks'}>\n".
					"<input type=hidden name=type value=$in{'type'}>\n".
					"<p><center><input type=submit value=\"Save Changes\"></center></p></form>\n";
		}
		elsif ($in{'specificSubmission'} && !$INFO{'shownLinks'}) {	
			print "<p align=center><small><b>No submission to review</b></small></p></form>\n";
		}
		else {
			print "<p align=center><small><b>No $in{'type'} Submissions to review</b></small></p></form>\n";
		}

		&printTemplate($CONF{'scriptWorkingDir'} . "templates/template.AdminFooter.txt");
		exit;


	}

	elsif ($processAction eq "save") {
		# We do not check the validity of anything as we allow the webmaster full control over
		# what is entered into the database. All required fields are in hidden input tags and are
		# not editable.

		# We step through each of the links and process them in turn
		for ($x=1; $x <= $in{'shownLinks'}; $x++) {
			# Default values
			$in{'reviewer' . $x} = $in{'thisReviewer'};

			undef($thisLinkData);

			# If the link is left Pending then we skip it
			if ($in{'siteStatus' . $x} eq "Pending") { next; }

			# If the link is Accepted then we process it
			elsif ($in{'siteStatus' . $x} eq "Accepted") {
				# Default values
				$in{'listStatus' . $x} = "Valid";
				$in{'linkbotStatus' . $x} = "200";
				$in{'timeReviewed' . $x} = time();
				$in{'reviewer' . $x} = $in{'thisReviewer'};

				# Prepare the sort name (make it lower case, replace numbers with words and remove all none alpha characters)
				$in{'sortName' . $x} = $in{'linkName' . $x};
				$in{'sortName' . $x} =~ tr/A-Z/a-z/;
				$in{'sortName' . $x} =~ s/^the //ig;
				$in{'sortName' . $x} =~ s/([^a-z0-9]*)//g;
				$in{'sortName' . $x} =~ s/1/one/g;
				$in{'sortName' . $x} =~ s/2/two/g;
				$in{'sortName' . $x} =~ s/3/three/g;
				$in{'sortName' . $x} =~ s/4/four/g;
				$in{'sortName' . $x} =~ s/5/five/g;
				$in{'sortName' . $x} =~ s/6/six/g;
				$in{'sortName' . $x} =~ s/7/seven/g;
				$in{'sortName' . $x} =~ s/8/eight/g;
				$in{'sortName' . $x} =~ s/9/nine/g;
				$in{'sortName' . $x} =~ s/0/zero/g;

				# Step through the fields and put the information into one string
				for ($y=0; $y < scalar(@linksFieldOrder); $y++) {
					$fieldName = @linksFieldOrder[$y];
					$thisLinkData .= $in{$fieldName . $x} . "\t";
				}
				# Get rid of the extra tab
				chop($thisLinkData);

				# Add the link to the Accepted Links file
				&addLink($thisLinkData);

				# If send email is selected, then shoot off an email
				if ($in{'sendEmail' . $x}) { &sendEmail("EmailAccept",$x); }
			}

			# If the link is Rejected then we process it
			elsif ($in{'siteStatus' . $x} eq "Rejected") { 
				# Default values
				$in{'timeReviewed' . $x} = time();

				# Step through the fields and put the information into one string
				for ($y=0; $y < scalar(@linksFieldOrder); $y++) {
					$fieldName = @linksFieldOrder[$y];
					$thisLinkData .= $in{$fieldName . $x} . "\t";
				}
				# Get rid of the extra tab
				chop($thisLinkData);

				# Add the link to the Rejected Links file
				&rejectLink($thisLinkData);

				# If send email is selected, then shoot off an email
				if ($in{'sendEmail' . $x}) { &sendEmail("EmailReject",$x); }
			}

			# Log the link as having been reviewed
			$thisLinkNumber = $in{'linkNumber' . $x};
			$linkReviewed{$thisLinkNumber} = 1;
		}

		# Remove the submissions which have been reviewed
		&removeSubmissions;

		# Redirect the reviewer to the review page
		print "Location: $ENV{'HTTP_REFERER'}\n\n";
		exit;

	}

	if ($processAction eq "logout") {
	
		&clearExclusions;

		$INFO{'showMessage'} = "<p><table cellspacing=0 cellpadding=0 border=0 align=center width=600><tr><td valign=top width=35 align=left><img src=\"$CONF{'laImageDir'}tickWhite.gif\" height=29 width=29></td><td><font size=2><b>Logged Out</b><br>You have been logged out and any outstanding submissions given to you are now available to other reviewers.</td></tr></table></p>\n";

		print "Content-type: text/html\n\n";
		&printTemplate($CONF{'scriptWorkingDir'} . "templates/template.AdminHeader.txt");
		print $INFO{'showMessage'};
		&printTemplate($CONF{'scriptWorkingDir'} . "templates/template.AdminFooter.txt");
		exit;

	}

}




###############################################################
#                     SEARCH SUB ROUTINE                      #
###############################################################
sub processSearch {
	$matchesPerPage = 10;
	if ($processAction eq "search") {

		# Load in the module for the specific type of data storage being used
		require ($CONF{'scriptWorkingDir'} . "library/module." . $CONF{'dataStorage'} . ".pl");

		if (!$in{'keywords'} || $in{'keywords'} !~ /.{2}/) { 
			$errorResponse = "<li>".$englishErrorMessages{'shortSearch'}."\n";
			&printErrorMessage($englishErrorMessages{"searchErrorPrefix"},$errorResponse); exit;
		}

		($totalHits,@searchResults) = &searchLinks;
		$INFO{'searchResultsInfo'} = "<p>Found <b>$totalHits</b> matches for \"$in{'keywords'}\". ";
		if ($totalHits) {
			$INFO{'searchResultsInfo'} .= 	"Displaying matches <b>".
										((($in{'page'} - 1) * $matchesPerPage) + 1).
										"</b> to <b>".
										((($in{'page'} - 1) * $matchesPerPage) + scalar(@searchResults)).
										"</b>.";
			$totalResultPages = int($totalHits / $matchesPerPage) + 1;
			if ($totalResultPages * $matchesPerPage == $totalHits + $matchesPerPage) { $totalResultPages--; }
			if ($totalResultPages > 1) {
				$INFO{'searchResultsInfo'} .= "<br><small>Result Pages:</small> &nbsp;";
				for ($x=1; $x <= $totalResultPages; $x++) {
					$pageLink = $ENV{'REQUEST_URI'};
					$pageLink =~ s/page=([0-9]*)/page=$x/;
					if ($x != $in{'page'}) {
						$INFO{'searchResultsInfo'} .= "<a href=\"$pageLink\"><b>$x</b></a>&nbsp;&nbsp;";
					}
					else { $INFO{'searchResultsInfo'} .= "$x&nbsp;&nbsp;"; }
				}
			}

			$INFO{'searchResultsInfo'} .= "</p><p><table border=0 width=600 cellpadding=0 cellspacing=0 align=center><tr><td height=3></td></tr><tr><td bgcolor=#2255a0 height=1><table border=0 width=600 cellpadding=0 cellspacing=0><tr><td></td></tr></table></td></tr><tr><td height=10></td></tr></table>\n";
			# Multi reject form
			$INFO{'searchResults'} = "<form action=\"../MultiReject/save\" method=POST>\n";

			# Load in the template for the result
			open (RESULTTEMPLATE,$CONF{'scriptWorkingDir'} . "templates/template.SearchResult.txt");
			while (<RESULTTEMPLATE>) { $origResultFormat .= $_; }
			close (RESULTTEMPLATE);

			foreach $searchResult (@searchResults) {
				# Clear various values
				undef($INFO{'timeReviewed'});
				undef($INFO{'linkSubCategory'});
				undef($INFO{'modifyLink'});
				undef($INFO{'multiReject'});
				
				$resultFormat = $origResultFormat;
				@linkData = split (/\t/,$searchResult);
				for ($x=0; $x < scalar(@linksFieldOrder); $x++) {
					$DATA{@linksFieldOrder[$x]} = @linkData[$x];
				}

				if (!$DATA{'linkName'}) { $DATA{'linkName'} = $DATA{'siteName'}; }
				if (!$DATA{'linkURL'}) { $DATA{'linkURL'} = $DATA{'siteURL'}; }
				if (!$DATA{'siteStatus'}) {
					$DATA{'siteStatus'} = "Pending";
					$INFO{'modifyLink'} = "<p><a href=\"../Review/view?type=General&specificSubmission=$DATA{'linkNumber'}\" target=\"modifyNew\" onMouseOver=\"status='Review This Submission'; return true;\" onMouseOut=\"status='';\"\"><img src=\"$CONF{'laImageDir'}modify.gif\" width=20 height=20 alt=Modify border=0 hspace=3 alt=\"Review This Submission\"></a></p>";
				}
				else {
					$INFO{'modifyLink'} = "<p><a href=\"../Modify/view?siteStatus=$DATA{'siteStatus'}&linkNumber=$DATA{'linkNumber'}\" target=\"modifyNew\" onMouseOver=\"status='Modify This Link'; return true;\" onMouseOut=\"status='';\"\"><img src=\"$CONF{'laImageDir'}modify.gif\" width=20 height=20 alt=Modify border=0 alt=\"Modify This Link\"></a></p>";
				}
				if ($DATA{'siteStatus'} eq "Accepted" || $DATA{'siteStatus'} eq "Pending") {
					$INFO{'multiReject'} = "<p><input type=checkbox name=\"multiReject:$DATA{'linkNumber'}\" value=\"$DATA{'siteStatus'}\"></p>";
				}
				
				$INFO{'timeSubmitted'} = &formatTimestamp($DATA{'timeSubmitted'},"datetime");
				if ($DATA{'timeReviewed'}) { $INFO{'timeReviewed'} = &formatTimestamp($DATA{'timeReviewed'},"datetime"); }

				if ($DATA{'linkSubCategory'}) { $INFO{'linkSubCategory'} = "(<b>$DATA{'linkSubCategory'}</b>)"; }
				elsif ($DATA{'siteSubCategory'}) { $INFO{'linkSubCategory'} = "(<b>$DATA{'siteSubCategory'}</b>)"; }
				if (!$DATA{'linkCategory'}) { $DATA{'linkCategory'} = $DATA{'siteCategory'}; }

				while ($resultFormat =~ /%%([a-z]+):([a-z]+):([a-z]*):([a-z0-9]*)%%/ig) {
					undef $toReplaceWith;
					$insertDataType = $1;
					$insertDataKey = $2;
					$insertFieldType = $3;
					$insertFieldValue = $4;
					if ($insertDataType =~ /^CONF/) { 
						if ($insertFieldType eq "checkbox") { if ($CONF{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "checked"; } }
						else { $toReplaceWith = $CONF{$insertDataKey}; }
					}
					elsif ($insertDataType =~ /^INFO/) { 
						if ($insertFieldType eq "checkbox") { if ($INFO{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "checked"; } }
						else { $toReplaceWith = $INFO{$insertDataKey}; }
					}
					elsif ($insertDataType =~ /^DATA/) { 
						if ($insertFieldType eq "checkbox") { if ($DATA{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "checked"; } }
						else { $toReplaceWith = $DATA{$insertDataKey}; }
					}
					$resultFormat =~ s/%%$insertDataType:$insertDataKey:$insertFieldType:$insertFieldValue%%/$toReplaceWith/g;
				}
				$INFO{'searchResults'} .= $resultFormat;
			}

		}

	}

	# Multi reject form
	if ($totalHits) { $INFO{'searchResults'} .= "<tr><td colspan=7 align=center><input type=submit value=\"Reject Selected Sites\"></form></td></tr>\n"; }

	$INFO{'siteCategory'} = &getCategoryList($in{'siteCategory'});

	if (-e $CONF{'scriptWorkingDir'} . "data/data.SubCats.txt" && (stat($CONF{'scriptWorkingDir'} . "data/data.SubCats.txt")) [7] > 0) { 
		open (SUBCATS,$CONF{'scriptWorkingDir'} . "data/data.SubCats.txt"); 
		while (<SUBCATS>) { chop($_); push (@subCategories,$_); }
		close (SUBCATS);
		$INFO{'siteSubCategory'} = "<select name=siteSubCategory>\n<option value=\"\">All Link Sub Categories</option>";
		foreach $subCat (@subCategories) { 
			$INFO{'siteSubCategory'} .= "<option";
			if ($subCat eq $in{'siteSubCategory'}) { $INFO{'siteSubCategory'} .= " selected"; }
			$INFO{'siteSubCategory'} .= ">$subCat</option>";
		}
		$INFO{'siteSubCategory'} .= "</select>";
	}

	$INFO{'keywords'} = $in{'keywords'};
	$INFO{'siteStatus'} = $in{'siteStatus'};
	if ($in{'searchType'}) { $INFO{'searchType'} = $in{'searchType'}; }
	else { $INFO{'searchType'} = "string"; }

	&displayPage("AdminSearch",$INFO{'showMessage'});
}


###############################################################
#                 SHOW RAW DETAIL SUB ROUTINE                 #
###############################################################
sub processDetail {
	# Load in the module for the specific type of data storage being used
	require ($CONF{'scriptWorkingDir'} . "library/module." . $CONF{'dataStorage'} . ".pl");

	(@linkData) = &getDetail($in{'siteStatus'},$in{'linkNumber'});
	$DATA{'linkNumber'} = $in{'linkNumber'};

	$x=0;
	foreach $fieldName (@linksFieldOrder) {
		if (($fieldName eq "timeSubmitted" || $fieldName eq "timeReviewed") && @linkData[$x]) { @linkData[$x] = &formatTimestamp(@linkData[$x],"datetime"); }
		$INFO{'linkDetail'} .= "<tr><td bgColor=#6699e0><small><font color=#FFFFFF>$fieldName</small></td><td><small><font color=#FFFFFF>@linkData[$x]</small></td></tr>";
		$x++;
	}
		
	&displayPage("AdminDetail",$INFO{'showMessage'});
}


###############################################################
#                    MODIFY LINK SUB ROUTINE                   #
###############################################################
sub processModify {
	# Load in the module for the specific type of data storage being used
	require ($CONF{'scriptWorkingDir'} . "library/module." . $CONF{'dataStorage'} . ".pl");

	if ($processAction eq "save") {
		# Set up some of the values
		# If the status has changes, log the new review date and reviewer
		if ($in{'originalStatus'} ne $in{'siteStatus'}) { $in{'timeReviewed'} = time(); }
		if ($in{'siteStatus'} eq "Accepted" && !$in{'linkbotStatus'}) { $in{'linkbotStatus'} = "200"; }
		# Prepare the sort name (make it lower case, replace numbers with words and remove all none alpha characters)
		$in{'sortName'} = $in{'linkName'};
		$in{'sortName'} =~ tr/A-Z/a-z/;
		$in{'sortName'} =~ s/^the //ig;
		$in{'sortName'} =~ s/([^a-z0-9]*)//g;
		$in{'sortName'} =~ s/1/one/g;
		$in{'sortName'} =~ s/2/two/g;
		$in{'sortName'} =~ s/3/three/g;
		$in{'sortName'} =~ s/4/four/g;
		$in{'sortName'} =~ s/5/five/g;
		$in{'sortName'} =~ s/6/six/g;
		$in{'sortName'} =~ s/7/seven/g;
		$in{'sortName'} =~ s/8/eight/g;
		$in{'sortName'} =~ s/9/nine/g;
		$in{'sortName'} =~ s/0/zero/g;

		$submissionError = 0;

		# There are only 2 required fields for the new link
		if ($in{'siteStatus'} eq "Accepted") {
			@requiredFields = ("linkName","linkURL");
			foreach (@requiredFields) { if (!$in{$_}) { $submissionError = 1; $errorResponse = "<li>".$englishErrorMessages{'newLinkFieldsRequired'}."\n"; }}
			if ($submissionError) { &printErrorMessage($englishErrorMessages{"modifylinkErrorPrefix"},$errorResponse); exit; }
		}

		# Step through the fields and put the information into one string
		for ($y=0; $y < scalar(@linksFieldOrder); $y++) {
			$fieldName = @linksFieldOrder[$y];
			$thisLinkData .= $in{$fieldName} . "\t";
		}
		# Get rid of the extra tab
		chop($thisLinkData);

		# Update the link
		&updateLink($thisLinkData,$in{'linkNumber'},$in{'originalStatus'},$in{'siteStatus'});

		$INFO{'showMessage'} = "<p><table cellspacing=0 cellpadding=0 border=0 align=center width=600><tr><td valign=top width=35 align=left><img src=\"$CONF{'laImageDir'}tickWhite.gif\" height=29 width=29></td><td><font size=2><b>Link Updated</b><br>The link has been modified.</td></tr></table></p>\n";

		print "Content-type: text/html\n\n";
		&printTemplate($CONF{'scriptWorkingDir'} . "templates/template.AdminHeader.txt");
		print $INFO{'showMessage'};
		&printTemplate($CONF{'scriptWorkingDir'} . "templates/template.AdminFooter.txt");
		exit;

	}

	elsif ($processAction eq "view") {

		(@linkData) = &getDetail($in{'siteStatus'},$in{'linkNumber'});

		# Step through the fields and get the information
		for ($y=0; $y < scalar(@linksFieldOrder); $y++) {
			$fieldName = @linksFieldOrder[$y];
			$DATA{$fieldName} = @linkData[$y];
		}

		if (!$DATA{'listStatus'}) { $DATA{'listStatus'} = "Invalid"; }
		# If it is a new submission, we "fudge" the data presentation a bit
		if ($DATA{'siteStatus'} eq "Pending") {
			$DATA{'linkName'} = $DATA{'siteName'} unless $DATA{'linkName'};
			$DATA{'linkURL'} = $DATA{'siteURL'} unless $DATA{'linkURL'};
			$DATA{'linkCategory'} = $DATA{'siteCategory'} unless $DATA{'linkCategory'};
			$DATA{'linkSubCategory'} = $DATA{'siteSubCategory'} unless $DATA{'linkSubCategory'};
		}

		$INFO{'linkCategory'} = &getCategoryList($DATA{'linkCategory'});

		# Get the sub category names
		if (-e $CONF{'scriptWorkingDir'} . "data/data.SubCats.txt" && (stat($CONF{'scriptWorkingDir'} . "data/data.SubCats.txt")) [7] > 0) { 
			open (SUBCATS,$CONF{'scriptWorkingDir'} . "data/data.SubCats.txt"); 
			while (<SUBCATS>) { chop($_); push (@subCategories,$_); }
			close (SUBCATS);

			$INFO{'linkSubCategory'} = "<tr><td bgColor=\"#6699e0\"><font size=2 color=\"#FFFFFF\"><b>Link Sub Category:</b></font></td><td><font size=2 color=\"#FFFFFF\"><select name=linkSubCategory>";
			foreach $subCat (@subCategories) { 
				$INFO{'linkSubCategory'} .= "<option";
				if ($subCat eq $DATA{'linkSubCategory'}) { $INFO{'linkSubCategory'} .= " selected"; }
				$INFO{'linkSubCategory'} .= ">$subCat</option>";
			}
			$INFO{'linkSubCategory'} .= "</select></font></td></tr>";
		}
		&displayPage("AdminModify",$INFO{'showMessage'});
	}


}


###############################################################
#                   REJECT MULTIPLE LINKS                     #
###############################################################
sub processMultiReject {
	# Load in the module for the specific type of data storage being used
	require ($CONF{'scriptWorkingDir'} . "library/module." . $CONF{'dataStorage'} . ".pl");

	# There is only one option for this function and that is to remove them
	# We need to read through the data sent in and extract the links and their
	# original status
	while (($key,$val) = each %in) {
		if ($key =~ /^multiReject/) {
			($d,$linkNumber) = split (/:/,$key);
			push (@multiReject,$linkNumber);
			$fileToProcess{$val} = 1;
		}
	}

	$multiRejectResult = &multiReject;

	$INFO{'showMessage'} = "<p><table cellspacing=0 cellpadding=0 border=0 align=center width=600><tr><td valign=top width=35 align=left><img src=\"$CONF{'laImageDir'}tickWhite.gif\" height=29 width=29></td><td><font size=2><b>Link(s) Rejected</b><br>$multiRejectResult links put to \"Rejected\" status.</td></tr></table></p>\n";

	print "Content-type: text/html\n\n";
	&printTemplate($CONF{'scriptWorkingDir'} . "templates/template.AdminHeader.txt");
	print $INFO{'showMessage'};
	&printTemplate($CONF{'scriptWorkingDir'} . "templates/template.AdminFooter.txt");
	exit;

}


###############################################################
#                    ADD LINK SUB ROUTINE                     #
###############################################################
sub processAdd {
	if ($processAction eq "save") {
		$submissionError = 0;

		# There are only 2 required fields for the new link
		@requiredFields = ("linkName","linkURL");
		foreach (@requiredFields) { if (!$in{$_}) { $submissionError = 1; $errorResponse = "<li>".$englishErrorMessages{'newLinkFieldsRequired'}."\n"; }}
		if ($submissionError) { &printErrorMessage($englishErrorMessages{"addlinkErrorPrefix"},$errorResponse); exit; }

		# Load in the module for the specific type of data storage being used
		require ($CONF{'scriptWorkingDir'} . "library/module." . $CONF{'dataStorage'} . ".pl");

		# We're this far so we save it
		$in{'recipStatus'} = "waived";
		$in{'linkNumber'} = &getLinkNumber;
		$in{'siteStatus'} = "Accepted";
		$in{'listStatus'} = "Valid";
		$in{'linkbotStatus'} = "200";
		$in{'timeSubmitted'} = time();
		$in{'timeReviewed'} = time();

		# Prepare the sort name (make it lower case, replace numbers with words and remove all none alpha characters)
		$in{'sortName' . $x} = $in{'linkName' . $x};
		$in{'sortName' . $x} =~ tr/A-Z/a-z/;
		$in{'sortName' . $x} =~ s/^the //ig;
		$in{'sortName' . $x}=~ s/([^a-z0-9]*)//g;
		$in{'sortName' . $x} =~ s/1/one/g;
		$in{'sortName' . $x} =~ s/2/two/g;
		$in{'sortName' . $x} =~ s/3/three/g;
		$in{'sortName' . $x} =~ s/4/four/g;
		$in{'sortName' . $x} =~ s/5/five/g;
		$in{'sortName' . $x} =~ s/6/six/g;
		$in{'sortName' . $x} =~ s/7/seven/g;
		$in{'sortName' . $x} =~ s/8/eight/g;
		$in{'sortName' . $x} =~ s/9/nine/g;
		$in{'sortName' . $x} =~ s/0/zero/g;


		# Step through the fields and put the information into one string
		for ($y=0; $y < scalar(@linksFieldOrder); $y++) {
			$fieldName = @linksFieldOrder[$y];
			$thisLinkData .= $in{$fieldName} . "\t";
		}
		# Get rid of the extra tab
		chop($thisLinkData);

		# Add the link to the Accepted Links file
		&addLink($thisLinkData);

		$CONF{'blockMessage'} = $in{'blockMessage'};
		$INFO{'showMessage'} = "<p><table cellspacing=0 cellpadding=0 border=0 align=center width=600><tr><td valign=top width=35 align=left><img src=\"$CONF{'laImageDir'}tickWhite.gif\" height=29 width=29></td><td><font size=2><b>Link Added</b><br>The new link has been added.</td></tr></table></p>\n";
	}

	$INFO{'linkCategory'} = &getCategoryList();

	# Get the sub category names
	if (-e $CONF{'scriptWorkingDir'} . "data/data.SubCats.txt" && (stat($CONF{'scriptWorkingDir'} . "data/data.SubCats.txt")) [7] > 0) { 
		open (SUBCATS,$CONF{'scriptWorkingDir'} . "data/data.SubCats.txt"); 
		while (<SUBCATS>) { chop($_); push (@subCategories,$_); }
		close (SUBCATS);

		$INFO{'linkSubCategory'} = "<tr><td bgColor=\"#6699e0\"><font size=2 color=\"#FFFFFF\"><b>Link Sub Category:</b></font></td><td><font size=2 color=\"#FFFFFF\"><select name=linkSubCategory>";
		foreach $subCat (@subCategories) { 
			$INFO{'linkSubCategory'} .= "<option>$subCat</option>";
		}
		$INFO{'linkSubCategory'} .= "</select></font></td></tr>";
	}
	&displayPage("AdminAdd",$INFO{'showMessage'});
}


###############################################################
#                    BLOCKED SUB ROUTINE                      #
###############################################################
sub processBlocked {
	if ($processAction eq "save") {
		$submissionError = 0;

		# There are no required fields for the Blocked items
		@tempBlockedItems = split (/\n/,$in{'blockedItems'});

		$blockCount = 0;

		open (BLOCKED,">$CONF{'scriptWorkingDir'}data/data.Blocked.txt"); &wLock('BLOCKED');
		foreach (@tempBlockedItems) {
			# Prep the info - trimming all white space
			$_ =~ s/^\s*(.*?)\s*$/$1/;
			if ($_) { print BLOCKED "$_\n"; }
			$blockCount++;
		}
		&unLock('BLOCKED'); close (BLOCKED);

		$CONF{'blockMessage'} = $in{'blockMessage'};
		$INFO{'showMessage'} = "<p><table cellspacing=0 cellpadding=0 border=0 align=center width=600><tr><td valign=top width=35 align=left><img src=\"$CONF{'laImageDir'}tickWhite.gif\" height=29 width=29></td><td><font size=2><b>Changes Saved</b><br>Your Blocked List has been updated. It contains <b>$blockCount</b> item(s).</td></tr></table></p>\n";
	}
	open (BLOCKED,$CONF{'scriptWorkingDir'} . "data/data.Blocked.txt"); 
	@blockedItems = (<BLOCKED>);
	close (BLOCKED);

	foreach (@blockedItems) { $DATA{'blockedItems'} .= $_; }
	&displayPage("AdminBlocked",$INFO{'showMessage'});
}


###############################################################
#                   CATEGORIES SUB ROUTINE                    #
###############################################################
sub processCategories {
	if ($processAction eq "save") {
		$submissionError = 0;
	
		# Check there are pages specified for each Category Name
		for ($x=1;$x<=$in{'catCount'};$x++) {
			if ($in{'otherCatPage'.$x}) { $in{'categoryPage'.$x} = $in{'otherCatPage'.$x}; } 
			if ($in{'categoryName'.$x} && !$in{'categoryPage'.$x} && $x > 0) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{'blankPageForCatName'}."\n"; $x=$in{'catCount'}; }
		}
	
		# Check there are pages specified for each Category Name
		for ($y=1;$y<=$in{'catCount'};$y++) {
			if ($in{'categoryName'.$y} && $catNameUsed{$in{'categoryName'.$y}}) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{'duplicateCatName'}."\n"; $y=$in{'catCount'}}
			else { $catNameUsed{$in{'categoryName'.$y}} = 1; }
			if ($in{'categoryPage'.$y} && $catPageUsed{$in{'categoryPage'.$y}}) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{'duplicateCatPage'}."\n"; $y=$in{'catCount'}}
			else { $catPageUsed{$in{'categoryPage'.$y}} = 1; }
			if ($in{'categoryName'.$y}) { $newCats .= "$in{'categoryName'.$y}\t$in{'categoryPage'.$y}\n"; $catCount++; }
		}
		if ($submissionError) { &printErrorMessage($englishErrorMessages{"categoriesErrorPrefix"},$errorResponse); exit; }
	
		# Save the new category listing
		open (CATEGORIES,">$CONF{'scriptWorkingDir'}data/data.Categories.txt"); &wLock('CATEGORIES');
		print CATEGORIES $newCats;
		&unLock('CATEGORIES'); close (CATEGORIES);
	
		# There are no required fields for the Sub Categories
		@tempSubCategories = split (/\n/,$in{'subCategories'});
	
		$subCatCount = 0;
	
		open (SUBCATS,">$CONF{'scriptWorkingDir'}data/data.SubCats.txt"); &wLock('SUBCATS');
		foreach (@tempSubCategories) {
			# Prep the info - trimming all white space
			$_ =~ s/^\s*(.*?)\s*$/$1/;
			if ($_) { print SUBCATS "$_\n"; }
			$subCatCount++;
		}
		&unLock('SUBCATS'); close (SUBCATS);

		$CONF{'newLinksPage'} = $in{'newLinksPage'};
		$CONF{'catListPage'} = $in{'catListPage'};

		$INFO{'showMessage'} = "<p><table cellspacing=0 cellpadding=0 border=0 align=center width=600><tr><td valign=top width=35 align=left><img src=\"$CONF{'laImageDir'}tickWhite.gif\" height=29 width=29></td><td><font size=2><b>Changes Saved</b><br>Your Categories and Sub-Categories lists have been updated with <b>$catCount</b> Categories and <b>$subCatCount</b> Sub-Categories.</td></tr></table></p>\n";
	}
	opendir (DIR,$CONF{'baseHTMLPath'});
	@allFiles = grep /.\.([a-z]*)$/i, readdir DIR;
	closedir (DIR);
	@allFiles = sort (@allFiles);
	foreach (@allFiles) { if (-T "$CONF{'baseHTMLPath'}$_") { push (@pageList,$_); } }

	open (CATEGORIES,$CONF{'scriptWorkingDir'} . "data/data.Categories.txt"); 
	@categoryList = (<CATEGORIES>);
	close (CATEGORIES);

	$INFO{'catCount'} = 1;

	foreach (@categoryList) { 
		chop ($_);
		($DATA{'categoryName'},$DATA{'categoryPage'}) = split (/\t/,$_);
		$INFO{'categoryList'} .= "<tr><td><font size=2><input type=text name=categoryName$INFO{'catCount'} value=\"$DATA{'categoryName'}\" size=15></td>";
		$INFO{'categoryList'} .= "<td><font size=2>" . &generateCatPageDropDown($DATA{'categoryPage'},$INFO{'catCount'}) . " <input type=text name=otherCatPage$INFO{'catCount'} size=15></font></td>\n";
		$INFO{'catCount'}++;
	}

	for ($w=0;$w<5;$w++) { 
		$INFO{'categoryList'} .= "<tr><td><font size=2><input type=text name=categoryName$INFO{'catCount'} size=15></td>";
		$INFO{'categoryList'} .= "<td><font size=2>" . &generateCatPageDropDown('',$INFO{'catCount'}) . " <input type=text name=otherCatPage$INFO{'catCount'} size=15></font></td>\n";
		$INFO{'catCount'}++;
	}

	open (SUBCATS,$CONF{'scriptWorkingDir'} . "data/data.SubCats.txt"); 
	@subCategories = (<SUBCATS>);
	close (SUBCATS);

	foreach (@subCategories) { $DATA{'subCategories'} .= $_; }
	&displayPage("AdminCategories",$INFO{'showMessage'});
}

sub generateCatPageDropDown {
	$toReturn = "<select name=categoryPage" . @_[1] . "><option value=\"\">Other, specify:</option>";
	undef $pageMatched;
	foreach $page (@pageList) {
		$toReturn .= "<option";
		if ($page eq @_[0]) { $toReturn .= " selected"; $pageMatched = 1; }
		$toReturn .= ">$page</option>";
	}
	if (@_[0] ne "" && !$pageMatched) { $toReturn .= "<option selected>" . @_[0] . "</option>"; }
	$toReturn .= "</select>";
	return $toReturn;
}


###############################################################
#                    TEMPLATES SUB ROUTINE                    #
###############################################################
sub processTemplates {
	if ($processAction eq "save") {
		$submissionError = 0;
	
		# We need to check the fields are completed
		@requiredFields = &getRequiredEmailTemplateFields;
		foreach (@requiredFields) { if (!$in{$_}) { $submissionError = 1; $errorResponse = "<li>".$englishErrorMessages{'allFieldsRequired'}."\n"; }}
	
		if ($submissionError) { &printErrorMessage($englishErrorMessages{"templateErrorPrefix"},$errorResponse); exit; }
	
		open (SUBMITTEMP,">$CONF{'scriptWorkingDir'}templates/template.EmailSubmit.txt"); &wLock('SUBMITTEMP');
		print SUBMITTEMP 	"To: %%WEBMASTEREMAIL%% (%%WEBMASTERNAME%%)\nFrom: ".
#		print SUBMITTEMP 	"To: greenguy\@link-o-rama.com (Greenguy)\nFrom: ".
							$in{'submitFrom'} . "\nSubject: " . $in{'submitSubject'} . "\n\n" . $in{'submitBody'};
		&unLock('SUBMITTEMP'); close (SUBMITTEMP);
	
		open (ACCEPTTEMP,">$CONF{'scriptWorkingDir'}templates/template.EmailAccept.txt"); &wLock('ACCEPTTEMP');
		print ACCEPTTEMP 	"To: %%WEBMASTEREMAIL%% (%%WEBMASTERNAME%%)\nFrom: ".
#		print ACCEPTTEMP 	"To: greenguy\@link-o-rama.com (Greenguy)\nFrom: ".
							$in{'acceptFrom'} . "\nSubject: " . $in{'acceptSubject'} . "\n\n" . $in{'acceptBody'};
		&unLock('ACCEPTTEMP'); close (ACCEPTTEMP);
	
		open (REJECTTEMP,">$CONF{'scriptWorkingDir'}templates/template.EmailReject.txt"); &wLock('REJECTTEMP');
		print REJECTTEMP 	"To: %%WEBMASTEREMAIL%% (%%WEBMASTERNAME%%)\nFrom: ".
#		print REJECTTEMP 	"To: greenguy\@link-o-rama.com (Greenguy)\nFrom: ".
							$in{'rejectFrom'} . "\nSubject: " . $in{'rejectSubject'} . "\n\n" . $in{'rejectBody'};
		&unLock('REJECTTEMP'); close (REJECTTEMP);
	
		$INFO{'showMessage'} = "<p><table cellspacing=0 cellpadding=0 border=0 align=center width=600><tr><td valign=top width=35 align=left><img src=\"$CONF{'laImageDir'}tickWhite.gif\" height=29 width=29></td><td><font size=2><b>Changes Saved</b><br>Your Email Templates have been updated.</td></tr></table></p>\n";
	}	
	open (SUBMITTEMP,$CONF{'scriptWorkingDir'} . "templates/template.EmailSubmit.txt"); 
	@submitEmail = (<SUBMITTEMP>);
	close (SUBMITTEMP);
	$DATA{'submitTo'} = "%%WEBMASTEREMAIL%% (%%WEBMASTERNAME%%)";
	$DATA{'submitFrom'} = @submitEmail[1]; chop ($DATA{'submitFrom'}); $DATA{'submitFrom'} =~ s/^From: //i;
	$DATA{'submitSubject'} = @submitEmail[2]; chop($DATA{'submitSubject'}); $DATA{'submitSubject'} =~ s/^Subject: //i;
	for ($x=4;$x<=scalar(@submitEmail);$x++) { $DATA{'submitBody'} .= @submitEmail[$x]; }

	open (ACCEPTTEMP,$CONF{'scriptWorkingDir'} . "templates/template.EmailAccept.txt"); 
	@acceptEmail = (<ACCEPTTEMP>);
	close (ACCEPTTEMP);
	$DATA{'acceptTo'} = "%%WEBMASTEREMAIL%% (%%WEBMASTERNAME%%)";
	$DATA{'acceptFrom'} = @acceptEmail[1]; chop ($DATA{'acceptFrom'}); $DATA{'acceptFrom'} =~ s/^From: //i;
	$DATA{'acceptSubject'} = @acceptEmail[2]; chop($DATA{'acceptSubject'}); $DATA{'acceptSubject'} =~ s/^Subject: //i;
	for ($y=4;$y<=scalar(@acceptEmail);$y++) { $DATA{'acceptBody'} .= @acceptEmail[$y]; }

	open (REJECTTEMP,$CONF{'scriptWorkingDir'} . "templates/template.EmailReject.txt"); 
	@rejectEmail = (<REJECTTEMP>);
	close (REJECTTEMP);
	$DATA{'rejectTo'} = "%%WEBMASTEREMAIL%% (%%WEBMASTERNAME%%)";
	$DATA{'rejectFrom'} = @rejectEmail[1]; chop ($DATA{'rejectFrom'}); $DATA{'rejectFrom'} =~ s/^From: //i;
	$DATA{'rejectSubject'} = @rejectEmail[2]; chop($DATA{'rejectSubject'}); $DATA{'rejectSubject'} =~ s/^Subject: //i;
	for ($z=4;$z<=scalar(@rejectEmail);$z++) { $DATA{'rejectBody'} .= @rejectEmail[$z]; }

	$INFO{'catCount'}--;
	&displayPage("AdminTemplates",$INFO{'showMessage'});
}


###############################################################
#                    REASONS SUB ROUTINE                      #
###############################################################
sub processReasons {
	if ($processAction eq "save") {
		$submissionError = 0;

		# There are no required fields for the Blocked items
		@tempRejectionReasons = split (/\n/,$in{'rejectionReasons'});

		$reasonCount = 0;

		open (REASONS,">$CONF{'scriptWorkingDir'}data/data.Reasons.txt"); &wLock('REASONS');
		foreach (@tempRejectionReasons) {
			# Prep the info - trimming all white space
			$_ =~ s/^\s*(.*?)\s*$/$1/;
			if ($_) { print REASONS "$_\n"; }
			$reasonCount++;
		}
		&unLock('REASONS'); close (REASONS);

		$INFO{'showMessage'} = "<p><table cellspacing=0 cellpadding=0 border=0 align=center width=600><tr><td valign=top width=35 align=left><img src=\"$CONF{'laImageDir'}tickWhite.gif\" height=29 width=29></td><td><font size=2><b>Changes Saved</b><br>Your Rejection Reasons have been updated. There are <b>$reasonCount</b> reason(s).</td></tr></table></p>\n";
	}
	open (REASONS,$CONF{'scriptWorkingDir'} . "data/data.Reasons.txt"); 
	@rejectionReasons = (<REASONS>);
	close (REASONS);

	foreach (@rejectionReasons) { $DATA{'rejectionReasons'} .= $_; }
	&displayPage("AdminReasons",$INFO{'showMessage'});
}


###############################################################
#                    CONFIGURE SUB ROUTINE                    #
###############################################################
sub processConfigure {
	if ($processAction eq "save") {
		$submissionError = 0;
	
		# Check all required fields are completed
		@requiredFields = &getRequiredConfigureFields;
		foreach (@requiredFields) {
			if (!$in{$_}) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{$_}."\n"; }
		}
	
		# Check that the email address provided appears valid
		if ($in{'adminEmail'} && !&checkEmailSyntax($in{'adminEmail'})) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"invalidEmail"}."\n"; }
	
		# Check that the redirect URL provided appears valid
		$redirectDomain = &parseURL($in{'redirectURL'});
		if ($in{'redirectURL'} && !&checkURLSyntax($redirectDomain)) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"invalidRedirectURL"}."\n"; }
	
		# Check that the submit script URL provided appears valid
		$submitDomain = &parseURL($in{'submitURL'});
		if ($in{'submitURL'} && !&checkURLSyntax($submitDomain)) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"invalidSubmitURL"}."\n"; }
	
		if ($in{'autoCheckRecip'}) {
			# Extract host for the Reciprocal Link URL
			$siteDomain = &parseURL($in{'checkRecipURL'});
			# Check syntax of site URL
			if ($in{'checkRecipURL'} && !&checkURLSyntax($siteDomain)) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"invalidRecipCheckURL"}."\n"; }
		}
	
		if ($in{'displayNewLink'} && $in{'newLinkDuration'}) {
			if ($in{'newLinkDuration'} =~ m|([^0-9])|) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"invalidNewLinkDuration"}."\n"; }
		}
	
		# Check the location of the Mail Program
		unless (-e $in{'mailProgram'}) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"invalidMailProgram"}."\n"; }
	
		# Check the location of the Base HTML Path
		if ($in{'baseHTMLPath'} !~ m|/$|) { $in{'baseHTMLPath'} .= "/";}
		unless (-e $in{'baseHTMLPath'} && -d $in{'baseHTMLPath'}) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"invalidBaseHTMLPath"}."\n"; }
	
		if ($in{'displayNewLink'} && $in{'newLinkDuration'}) {
			if ($in{'newLinkDuration'} =~ m|([^0-9])|) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"invalidNewLinkDuration"}."\n"; }
		}
	
		if ($submissionError) { &printErrorMessage($englishErrorMessages{"configureErrorPrefix"},$errorResponse); exit; }
	
		foreach (@configureFields) { $CONF{$_} = $in{$_}; }
		$INFO{'showMessage'} = "<p><table cellspacing=0 cellpadding=0 border=0 align=center width=600><tr><td valign=top width=35 align=left><img src=\"$CONF{'laImageDir'}tickWhite.gif\" height=29 width=29></td><td><font size=2><b>Changes Saved</b><br>Your configuration has been updated.</td></tr></table></p>\n";
	
	}

	# Read in the modules and load each of them up to get the version numbers
	opendir (DIR,$CONF{'scriptWorkingDir'} . "library");
	@allModules = grep /\.pl$/, readdir DIR;
	closedir (DIR);
	foreach $moduleName (@allModules) {
		if ($moduleName =~ /^module/i) { 
			require ($CONF{'scriptWorkingDir'} . "library/$moduleName");
			$moduleName =~ s/^module\.([a-z]*)\.pl/$1/i;
			$INFO{'moduleVersions'} .= 	"<tr><td bgColor=\"#6699e0\" width=50%><font size=2 color=\"#FFFFFF\">".
										"<b>$moduleName Module:</b></font></td>".
										"<td width=50%><font size=2 color=\"#FFFFFF\">".
										$DATA{"module." . $moduleName . ".Version"} . "</font></td></tr>";
		}
	}
	&displayPage("AdminConfigure",$INFO{'showMessage'});
}


###############################################################
#                    FORM HTML SUB ROUTINE                    #
###############################################################
sub processFormhtml {
	if ($CONF{'recipLink'}) { $INFO{'recipLink'} = "\n <tr>\n  <td>URL Of Page With Reciprocal Link:</td>\n  <td><input type=text name=recipURL size=30></td>\n </tr>"; }

	$INFO{'categories'} = &getCategoryList();

	if (-e $CONF{'scriptWorkingDir'} . "data/data.SubCats.txt" && (stat($CONF{'scriptWorkingDir'} . "data/data.SubCats.txt")) [7] > 0) { 
		$INFO{'subCategories'} = "\n <tr>\n  <td>Sub Category:</td>\n  <td>\n   <select name=siteSubCategory>";
		open (SUBCATS,$CONF{'scriptWorkingDir'} . "data/data.SubCats.txt"); 
		@subCategories = (<SUBCATS>);
		close (SUBCATS);
		foreach (@subCategories) { chop($_); unless ($_ =~ /^none$/i) { $INFO{'subCategories'} .= "\n    <option>$_</option>"; } }
		$INFO{'subCategories'} .= "\n   </select>\n  </td>\n </tr>";
	}
	&displayPage("AdminFormhtml","");
}


###############################################################
#                  REFRESH PAGES SUB ROUTINE                  #
###############################################################
sub processRefresh {

	if ($processAction eq "run") {
		require ($CONF{'scriptWorkingDir'} . "library/module.Refresh.pl");
		$refreshResult = &refreshPages;
		$INFO{'showMessage'} = "<p><table cellspacing=0 cellpadding=0 border=0 align=center width=600><tr><td valign=top width=35 align=left><img src=\"$CONF{'laImageDir'}tickWhite.gif\" height=29 width=29></td><td><p><font size=2><b>Pages Refreshed</b><br>Result from script:</font></p>\n<xmp>$refreshResult</xmp></td></tr></table></p>\n";
	}
	elsif ($processAction eq "import") {
		# Load in the module for the specific type of data storage being used
		require ($CONF{'scriptWorkingDir'} . "library/module." . $CONF{'dataStorage'} . ".pl");
		$importResult = &updateReviewedDate;
		$INFO{'showMessage'} = "<p><table cellspacing=0 cellpadding=0 border=0 align=center width=600><tr><td valign=top width=35 align=left><img src=\"$CONF{'laImageDir'}tickWhite.gif\" height=29 width=29></td><td><p><font size=2><b>Links Imported</b><br>$importResult links \"imported\".</font></p>\n</td></tr></table></p>\n";
	}

	open (IMPORTTIME,$CONF{'scriptWorkingDir'} . "data/data.LastImport.txt"); 
	$lastImport = (<IMPORTTIME>);
	close (IMPORTTIME);
	if ($lastImport) { $INFO{'lastImport'} = &formatTimestamp($lastImport,"datetime"); }
	else { $INFO{'lastImport'} = "never run"; }

	open (CATS,$CONF{'scriptWorkingDir'} . "data/data.Categories.txt"); 
	@categories = (<CATS>);
	close (CATS);

	@categories = sort(@categories);

	foreach $cat (@categories) { 
		($tempCat,$d) = split(/\t/,$cat);
		$INFO{'categoryRows'} .= 	"<tr><td bgColor=\"#6699e0\" width=50%><font size=2 color=\"#FFFFFF\"><b>$tempCat:</b></font></td>".
									"<td width=25%><font size=2 color=\"#FFFFFF\">Yes: <input type=radio name=\"ignoreRefresh:$tempCat\" value=0 checked></font></td>".
									"<td width=25%><font size=2 color=\"#FFFFFF\">No: <input type=radio name=\"ignoreRefresh:$tempCat\" value=1></font></td></tr>\n";
	}

	&displayPage("AdminRefresh","$INFO{'showMessage'}");
}


exit;