$DATA{'core.Library.Version'} = $CONF{'core.Library.Version'} = "1.2";

###############################################################
# Link Admin v 1.3 Core Library File                          #
###############################################################
# core.Library.pl v 1.3 RJ 03-Jan-2003                        #
# This contains shared functions and settings.                #
#                                                             #
# For further information please visit:                       #
# http://www.UsefulScripts.com/                               #
###############################################################
# Copyright  2001 iNet Publishing Ltd                        #
# Reproduction in part or in whole is strictly prohibited.    #
# This copyright header must not be removed.                  #
# No warranty expressed or implied.                           #
###############################################################

# Version 1.3 03-Jan-2003
# Modified processing and sending of emails to avoid incorrect details
# Version 1.2 16-Nov-2002
# Modified input parsing to trim white space and remove tabs
# Version 1.1 18-Feb-2002
# Modified URL and Email syntax checking to allow 4 character TLDs (e.g. .info).
# Version 1.0 13-Nov-2001
# Original script


###############################################################
#           DO NOT EDIT ANYTHING BENEATH THIS LINE!           #
###############################################################

# Pre-defined error responses 
%englishErrorMessages = (
	submitErrorPrefix =>		"Your submission could not be accepted for the following reason(s):",
	searchErrorPrefix =>		"The search could bot be completed for the following reason(s):",
	addlinkErrorPrefix =>		"The new link could not be added for the following reason(s):",
	modifylinkErrorPrefix =>	"The link could not be modified for the following reason(s):",
	configureErrorPrefix =>		"The changes to the configuration could not be saved for the following reason(s):",
	authErrorPrefix =>			"The changes to the authorisation restrictions could not be saved for the following reason(s):",
	unauthorisedErrorPrefix =>	"<b>Access Denied</b>:",
	categoriesErrorPrefix =>	"The changes to the categories could not be saved for the following reason(s):",
	templateErrorPrefix =>		"The changes to the email templates could not be saved for the following reason(s):",
	webmasterName => 			"Your Name is required.",
	webmasterEmail => 			"Your Email Address is required.",
	siteName => 				"The Site Name is required.",
	siteURL => 					"The Site URL is required.",
	siteCategory => 			"The Site Category is required.",
	siteSubCategory => 			"The Site Sub Category is required.",
	webmasterComments => 		"Your Comments are required.",
	recipURL => 				"A Reciprocal Link is required.",
	invalidEmail => 			"Your Email Address appears invalid.",
	invalidSiteURL => 			"Your Site's URL appears invalid.",
	invalidSiteResponse => 		"Your Site's URL appears invalid, as it has returned the response ",
	invalidRecipURL => 			"The URL for the page with the reciprocal link appears invalid.",
	invalidReferrer => 			"The form you completed is at an unauthorised URL for this program.",
	multiIPaddress => 			"You have already submitted a site within the last 24 hours.",
	duplicateURL =>				"The URL you have submitted is already listed or is awaiting review.",
	noRecipLink => 				"A reciprocal link is required and no reciprocal link was found.",
	blockedSite => 				"You, your site or your host has been blocked.",
	adminEmail => 				"Your Email Address is required.",
	redirectURL => 				"The Redirect URL is required.",
	linkOrder => 				"The Link Order is required.",
	linkFormatHTML => 			"The Link Format HTML is required.",
	newImageHTML => 			"The New Image HTML is required.",
	newImageDuration => 		"How long Images Are New is required.",
	hotImageHTML => 			"The Hot Image HTML is required.",
	submitScript => 			"The URL of the Submit Script is required.",
	mailProgram => 				"The location of the Mail Program is required.",
	baseHTMLPath => 			"The Base HTML Path is required.",
	invalidRedirectURL => 		"The Redirect URL appears invalid.",
	invalidSubmitURL => 		"The URL of the Submit Script appears invalid.",
	invalidRecipCheckURL => 	"The URL for reciprocal link appears invalid.",
	invalidNewLinkDuration =>	"How long Links Are New must be a number.",
	invalidMailProgram =>		"The location of the Mail Program is invalid.",
	invalidBaseHTMLPath =>		"The directory for the Base HTML Path is invalid.",
	authFieldsRequired =>		"At least one person must be authorised to access the Set Configuration Options and Set Authorisation Restrictions areas.",
	unauthorisedAccess =>		"You are not authorised to access this area.",
	blankPageForCatName =>		"There must be a page specified for each Category.",
	duplicateCatName =>			"All the Category Names must be unique.",
	duplicateCatPage =>			"A page can only be specified for one Category.",
	shortSearch =>				"The Keywords must be at least 2 characters.",
	allFieldsRequired =>		"All the fields are required and must be completed.",
	newLinkFieldsRequired =>	"The Link Name and Link URL are required.",
	submitAppendFailure =>		"Your submission could not be added to the database. Please try submitting again. If the problem persists please contact the <a href=\"mailto:$CONF{'adminEmail'}\">site admin</a>."
);

# Order of the data fields in the (all) links file
@linksFieldOrder = ("linkNumber","sortName","linkName","linkURL","recipURL","recipStatus","linkDescription","linkCategory","linkSubCategory","mouseoverText","hotLink","recommendLink","alwaysNew","expireAfter","siteName","siteURL","siteCategory","siteSubCategory","webmasterName","webmasterEmail","webmasterIPaddress","webmasterComments","rejectReason","friendSite","siteStatus","listStatus","linkbotStatus","linkbotComment","timeSubmitted","reviewer","timeReviewed");

# Fields for the Config file
@configureFields = ("adminEmail","siteName","redirectURL","checkDupURLs","reviewPageSites","recipLink","autoCheckRecip","checkRecipURL","submitEmailNotify","displayNewLink","displayNewLinkNewPage","newImageHTML","newLinkDuration","displayHotLink","hotImageHTML","displayRecommendLink","recommendImageHTML","siteNameFormat","siteDescriptionFormat","linkOrder","linkReverse","linkFormatHTML","submitScript","mailProgram","baseHTMLPath","laImageDir","defaultExpireAfter","multiIPBlocking","defaultEmailChecked");


###############################################################
#                     SHARED SUB ROUTINES                     #
###############################################################


# Compile the list of Required Configure Fields
sub getRequiredConfigureFields {
	# List of fields required for the initial submission
	@requiredFields = ("adminEmail","siteName","redirectURL","linkOrder","linkFormatHTML","submitScript","mailProgram","baseHTMLPath");
	if ($in{'autoCheckRecip'}) { push(@requiredFields,"checkRecipURL"); }
	if ($in{'displayNewLink'}) { push(@requiredFields,"newImageHTML"); }
	if ($in{'displayNewLink'}) { push(@requiredFields,"newLinkDuration"); }
	if ($in{'displayHotLink'}) { push(@requiredFields,"hotImageHTML"); }
	return @requiredFields;
}


# Compile the list of Required Configure Fields
sub getRequiredEmailTemplateFields {
	# List of fields required for the email templates
	@requiredFields = ("submitFrom","submitSubject","submitBody","acceptFrom","acceptSubject","acceptBody","rejectFrom","rejectSubject","rejectBody");
	return @requiredFields;
}


# Compile the list of Required Submission Fields
sub getRequiredSubmitFields {
	# List of fields required for the initial submission
	@requiredFields = ("webmasterName","webmasterEmail","siteName","siteURL","siteCategory","webmasterComments");
	if ($CONF{'recipLink'}) { push(@requiredFields,"recipURL"); }
	if (-e $CONF{'scriptWorkingDir'} . "data/data.SubCats.txt" && (stat($CONF{'scriptWorkingDir'} . "data/data.SubCats.txt")) [7] > 0) { push(@requiredFields,"siteSubCategory"); }
	return @requiredFields;
}

# Extract the Host from the URL
sub parseURL {
	$urlToParse = shift;
	if ($urlToParse !~ m|http://([^:/]*):?([0-9]*)/|) { $urlToParse .= "/"; }
	$urlToParse =~ m|http://([^:/]*):?([0-9]*)/|ig;
	return ($1);
}

# Very basic checking of URL syntax
sub checkURLSyntax {
	$urlToCheck = shift;
	if ( $urlToCheck =~ m|([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})|i) {
		if ($1<=255 && $2<=255 && $3<=255 && $4<=255) { return 1; } else { return 0; }
	}
	elsif ($urlToCheck =~ m|([a-z0-9-\.]+)\.([a-z]{2,4})$|i) { return 1; }
	else { return 0; }
}

# Very basic checking of Email syntax
sub checkEmailSyntax {
	$emailToCheck = shift;
	if ($emailToCheck =~ m|^.+\@([a-z0-9-\.]+)\.([a-z]{2,4})$|i) { return 1; } else { return 0; };
}

# Send an email
sub sendEmail {
	($template,$filter) = @_;
	
	while (($key,$value) = each %in) { 
		if ($key =~ /^([a-z]*)$filter$/i) {
			$newKey = $key; $newKey =~ tr/a-z/A-Z/; $newKey =~ s/$filter$//; $in{$newKey} = $value;
		}
	}	

	open (MAIL,"|$CONF{'mailProgram'} -t");
	open (MAILTEMPLATE,$CONF{'scriptWorkingDir'} . "templates/template.$template.txt"); 
	while (<MAILTEMPLATE>) {
		while ($_ =~ /%%([^%]*)%%/ig) {
			$toReplaceWith = $originalMatch = $1;
			$toReplaceWith =~ tr/a-z/A-Z/;
			$_ =~ s/%%$originalMatch%%/$in{$toReplaceWith}/g;
		}
		print MAIL $_;
	}
	close (MAILTEMPLATE);
	close (MAIL);
}


# File locking info
$LOCK_SH = 1;
$LOCK_EX = 2;
$LOCK_NB = 4;
$LOCK_UN = 8;

# Lock the file for writing
sub wLock {
	$thisFile = shift;
	if ($CONF{'fileLocking'}) {
		flock ($thisFile, $LOCK_EX);
		# If it's unavailable right now
		seek ($thisFile, 0, 2);
	}
}

# Lock the file for reading
sub rLock {
	$thisFile = shift;
	if ($CONF{'fileLocking'}) {
		flock ($thisFile, $LOCK_SH);
	}
}

# Unlock the file
sub unLock {
	$thisFile = shift;
	if ($CONF{'fileLocking'}) {
		flock ($thisFile, $LOCK_UN);
	}
}

# Parse the input from the form
sub parseFormInput {

	if ($ENV{'REQUEST_METHOD'} =~ /POST/i) {
		read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
		@pairs = split(/&/, $buffer);
	}
	elsif ($ENV{'REQUEST_METHOD'} =~ /GET/i) {
		@pairs = split(/&/, $ENV{'QUERY_STRING'});
	}
	foreach $pair (@pairs) {
		($name, $value) = split(/=/, $pair);
		$value =~ tr/+/ /;
		$name =~ tr/+/ /;
		$value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
		$name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
		$value =~ s/^\s*(.*?)\s*$/$1/;
		$value =~ s/\t//;
		$value =~ s/\cM//g;
		$name =~ s/\cM//g;
		unless ($name =~ /HTML$/) { $value =~ s/\"/&quot;/g; }
		$in{$name} = $value;
	}

	if ($ENV{'PATH_INFO'}) { ($d,$processModule,$processAction) = split (/\//,$ENV{'PATH_INFO'}); }

}

sub printErrorMessage {

	($errorPrefix,$errorResponse) = @_;

	print "Content-type: text/html\n\n";
	print <<"END";
	<html><head><title>Error!</title>
	<style Type="Text/CSS">
	<!--
		BODY,TD,P,LI	{font-family:Verdana}
		A				{font-family:Verdana;text-decoration: none}
		A:hover 		{text-decoration: underline}
	//-->
	</style></head>
	<body bgColor="#FFFFFF" Link="#0000FF" aLink="#FF0000" vLink="#828282" Text="#000000">
	<!-- Link Admin Script  2001 iNet Publishing Ltd  http://www.UsefulScripts.com/ -->
	<p><table cellspacing=0 cellpadding=0 border=0 align=center width=450>
	<tr><td valign=top><img src="
END
	print 	"$CONF{'laImageDir'}warningWhite.gif\" height=29 width=29>&nbsp;</td>\n".
			"<td><font size=2><b>Error!</b><br>";
	print $errorPrefix . "<ul>\n" . $errorResponse;
	print <<"END";
	</ul>\n<a href="javascript:history.go(-1)">Click here to return to the previous page.</a></font></td></tr>
	<tr><td></td><td><br><br><font size=1>Powered by Link Admin v
END
	if ($DATA{'core.Submit.Version'}) { print $DATA{'core.Submit.Version'}; }
	else { print $DATA{'core.Version'}; }
	print <<"END";
	from <a href="http://www.usefulscripts.com" target=_blank>UsefulScripts.com</a></td></tr></table></p>	
	<br><br><br><br><br><br><br></body></html>
END

#print "$responses - $pageHTML - $urlToTest"; 

	exit;

}

sub printTemplate {
	$thisPage = shift;
	open (PAGEBODY,$thisPage);
	while (<PAGEBODY>) {
		while ($_ =~ /%%([a-z]+):([a-z\._]+):([a-z]*):([a-z0-9]*)%%/ig) {
			undef $toReplaceWith;
			$insertDataType = $1;
			$insertDataKey = $2;
			$insertFieldType = $3;
			$insertFieldValue = $4;
			if ($insertDataType =~ /^CONF/) { 
				if ($insertFieldType eq "checkbox") { if ($CONF{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "checked"; } }
				elsif ($insertFieldType eq "selection") { if ($CONF{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "selected"; } }
				else { $toReplaceWith = $CONF{$insertDataKey}; }
			}
			elsif ($insertDataType =~ /^INFO/) {
				if ($insertFieldType eq "checkbox") { if ($INFO{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "checked"; } }
				elsif ($insertFieldType eq "selection") { if ($INFO{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "selected"; } }
				else { $toReplaceWith = $INFO{$insertDataKey}; }
			}
			elsif ($insertDataType =~ /^DATA/) {
				if ($insertFieldType eq "checkbox") { if ($DATA{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "checked"; } }
				elsif ($insertFieldType eq "selection") { if ($DATA{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "selected"; } }
				else { $toReplaceWith = $DATA{$insertDataKey}; }
			}
			elsif ($insertDataType =~ /^ENV/) {
				if ($insertFieldType eq "checkbox") { if ($ENV{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "checked"; } }
				elsif ($insertFieldType eq "selection") { if ($ENV{$insertDataKey} eq $insertFieldValue) { $toReplaceWith = "selected"; } }
				else { $toReplaceWith = $ENV{$insertDataKey}; }
			}
			$_ =~ s/%%$insertDataType:$insertDataKey:$insertFieldType:$insertFieldValue%%/$toReplaceWith/g;
		}
		print $_;
	}
	close (PAGEBODY);
}



sub displayPage {
	($thisTemplate,$INFO{'showMessage'}) = @_;
	print "Content-type: text/html\n\n";
	&printTemplate($CONF{'scriptWorkingDir'} . "templates/template.AdminHeader.txt");
	&printTemplate($CONF{'scriptWorkingDir'} . "templates/template.$thisTemplate.txt");
	&printTemplate($CONF{'scriptWorkingDir'} . "templates/template.AdminFooter.txt");
	exit;
}	


# Format a unix timestamp
sub formatTimestamp {
	@monthsShort = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
	@monthsLong = ('January','February','March','April','May','June','July','August','September','October','November','December');
	@weekDays = ('Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday');
	($sec,$min,$hour,$dy,$mn,$yr,$wdy,$d,$d) = localtime(@_[0]); $pyr=$yr+1900;
	$theTime{'time'} = ""; if ($hour < 10) {$theTime{'time'} .= "0";} $theTime{'time'} .= "$hour:"; if ($min < 10) {$theTime{'time'} .= "0";} $theTime{'time'} .= "$min:"; if ($sec < 10) {$theTime{'time'} .= "0";} $theTime{'time'} .= "$sec";
	$theTime{'date'} = "$dy @monthsShort[$mn] $pyr";
	$theTime{'longdate'} = "@weekDays[$wdy] $dy @monthsLong[$mn] $pyr";
	$theTime{'datetime'} = $theTime{'time'} . " on " . $theTime{'date'};

	return($theTime{@_[1]});
}	


sub getCategoryList {
	$thisCat = shift;
	undef($tempCatList);
	if (!@categories) {
		open (CATS,$CONF{'scriptWorkingDir'} . "data/data.Categories.txt"); 
		@categories = (<CATS>);
		close (CATS);
	}
	foreach $cat (@categories) { 
		($tempCat,$d) = split(/\t/,$cat);
		$tempCatList .= "<option";
		if ($tempCat eq $thisCat) { $tempCatList .= " selected"; }
		$tempCatList .= ">$tempCat</option>";
	}
	return $tempCatList;
}


1;
