#!/usr/bin/perl

###############################################################################
##
##	script		= spm.cgi (Simple Password Manager)
##	version		= 1.0
##	date		= 21/11/2001
##	author		= Richard Jones
##	copyright	=  Copyright iNet Publishing Ltd. 2001.  All Rights Reserved
##				  http://www.usefulscripts.com
##
##	Simple password manager script available for download from Useful Scripts.
##  http://www.usefulscripts.com
##
##  DESCRIPTION
##
##  This script allows password files (for use with htaccess) to be created and
##  maintained through a simple web interface.
##
##  SETUP INSTRUCTIONS
##
##  1)  Upload this file (spm.cgi) to a directory on your server.
##  2)  CHMOD this file on your server to 0755 (for more information on CHMOD
##      see http://www.usefulscripts.com/help/permissions.phtml).
##  3)  Create a subdirectory called "data" in the directory where you uploaded this
##      file.
##  4)  CHMOD the "data" directory created in step 3 to 0777 (for more information on CHMOD
##      see http://www.usefulscripts.com/help/permissions.phtml).
##  5)  Go to the script in your web browser and enter the username and password you
##      would like to use to access the script.
##  6)  Copy the text you are given after you have entered the username and password
##      and insert it into a new text and save that file as ".htaccess".
##  7)  Upload the ".htaccess" file to the same directory where you uploaded this
##      file.
##  8)  Go to the script in your web browser and create new password files and list,
##      add and remove usernames in password files.
##	
##	TERMS AND CONDITIONS OF USE:
##
##	This script is the intellectual property of the copyright owner named above
##	hereafter refered to as "The Owner".
##	"The User" refers to anyone who is using or planning to use the script for
##	any purpose whatsoever.
##	As such the following restrictions on usage apply:-
##
##	1)	The script is to be used entirely at the risk of the user.
##		No warranty is given or implied by the owner.
##	2)	The user is responsible for ensuring that the script is suitable for
##		the environment in which they intend to use it and accepts full
##		responsibility for any consequences arising from its use.
##	3)	The script may be copied, modified and/or distributed freely but the
##		copyright remains with the owner at all times and any further restrictions
##		applied by the user will not apply to the original script or the owner's
##		rights over it.
##	4)	The owner respectfully requests that the original copyright information
##		remains embeded in this script and that a link to Useful Scripts is placed
##		on any HTML pages related to it.  http://www.usefulscripts.com
##	5)	The owner reserves the right to change these terms and conditions without notice.
##		
##	VERSION INFO
##
##	1.0
##	- Original Script
##
###############################################################################


use Cwd;
$thisDir = cwd;
&printHeader;
if ($ENV{'QUERY_STRING'}) { &parseInput; }
$in{'pwf'} =~ s/\.pwf$//g;
unless ($ENV{'REMOTE_USER'} || $in{'function'} eq "Add") {
	# If the data directory does not exist
	unless (-e "./data") { &printResult('Starting up','Failed','The "data" directory does not exist. You need to create the subdirectory "data" in the directory where this script has been uploaded: "'.$thisDir.'"'); &printFooter; exit; }
	unless (-w "./data") { &printResult('Starting up','Failed','The "data" directory is not writable. You need to chmod the subdirectory "data" to "0766" (please see <a href="http://www.usefulscripts.com/help/permissions.phtml" target=_blank>http://www.usefulscripts.com/help/permissions.phtml</a> for information on chmod).'); &printFooter; exit; }
	unless (-e "./data/SPMadmin.pwf") { 
		&printResult('Starting up','Failed','You have not password protected this script. Please follow the instructions below.'); 
		print 	"<p><ol><li>Enter your chosen username and password to access this script in the form below and press the \"Add Username\" button.</li>\n".
				"<li>On the following page, copy the text in the box and place it in a text file called \".htaccess\".</li>\n".
				"<li>Upload the \".htaccess\" file to this directory on your server:<br>\"$thisDir\"</li></ol></p>\n";
		&addForm('SPMadmin'); 
		&printFooter; 
		exit; 
	}
	&printResult('Starting up','Failed','Copy the text in the box below and place it in a text file called ".htaccess". Upload the ".htaccess" file to this directory on your server:<br>"'.$thisDir.'"');
	&printHtaccessCode('SPMadmin');
	&printFooter; 
	exit;	

}
if ($in{'function'} eq "Add") {
	&addUsername;
	unless ($ENV{'REMOTE_USER'}) {
		print "<p><b>IMPORTANT:</b><br>Copy the text in the box above and place it in a text file called \".htaccess\".<br>Upload the \".htaccess\" file to this directory on your server:<br>\"$thisDir\"</p>\n";
		&printFooter; 
		exit;
	}
}
elsif ($in{'function'} eq "Remove") { &removeUsername; }
elsif ($in{'function'} eq "List") { &listUsernames($in{'pwf'}); }
elsif ($in{'function'} eq "New") { &newFile(); }
else { &addForm($in{'pwf'}); &listForm(); &fileForm(); }
&printFooter;
exit;


sub addUsername {
	$tempAction = "Adding username \"$in{'username'}\" and password \"$in{'password'}\" to file \"$in{'pwf'}\".";
	unless ($in{'username'} && $in{'password'}) { &printResult($tempAction,'Failed','Both a username and password are required.'); &addForm($in{'pwf'}); &printFooter; exit; }
	unless ($in{'pwf'}) { &printResult($tempAction,'Failed','A password file has not been specified.'); &addForm; &printFooter; exit; }
	$giveCode = 1;
	if (-e "./data/$in{'pwf'}.pwf") {
		open (PWF,"./data/$in{'pwf'}.pwf");
		while (<PWF>) {
			($tempUsername,$d) = split (/:/,$_);
			if ($tempUsername eq $in{'username'}) {
				&printResult($tempAction,'Failed','The username is already in the password file.');
				&addForm($in{'pwf'});
				&printFooter;
				close (PWF);
				exit;
			}
		}
		close (PWF);
		$giveCode = 0;
	}
	elsif ($in{'pwf'} eq "SPMadmin") { 
		open (HTFILE,">./data/.htaccess");
		print HTFILE	"AuthUserFile /dev/null\n".
						"AuthGroupFile /dev/null\n".
						"AuthName RESTRICTED\n".
						"AuthType Basic\n\n".
						"<Limit GET POST>\n".
						"require valid-user\n".
						"</Limit>\n";
		close (HTFILE);
	}

	$ePassword = crypt($in{'password'}, "Mx");
	open (PWF,">>./data/$in{'pwf'}.pwf");
	print PWF "$in{'username'}:$ePassword\n";
	close (PWF);
	&printResult($tempAction,'Success','');
	if ($giveCode) { &printHtaccessCode($in{'pwf'}); }
}


sub listUsernames {
	$thisPWF = shift;
	print	"<p><form action=\"$ENV{'SCRIPT_NAME'}\" METHOD=GET><input type=hidden name=function value=\"Remove\">\n".
			"Listing usernames for file \"$in{'pwf'}\":<br><br>\n";
	open (PWF,"./data/$in{'pwf'}.pwf");
	while (<PWF>) {
		($tempUsername,$d) = split (/:/,$_);
		print "<input type=checkbox name=remove value=\"$tempUsername\"> $tempUsername<br>\n";
	}
	close (PWF_);
	print "<br><input type=hidden name=pwf value=\"$in{'pwf'}\"><input type=submit value=\"Remove Selected Usernames\"></form></p>\n";
}


sub newFile {
	$tempAction = "Creating new password file \"$in{'pwf'}\".";
	if (-e "./data/$in{'pwf'}.pwf") { &printResult($tempAction,'Failed','The file "'.$in{'pwf'}.'" already exists. Please choose another name.'); &fileForm; &printFooter; exit; }
	else {
		open (PWF,">./data/$in{'pwf'}.pwf");
		close (PWF);
		&printResult($tempAction,'Success','');
		&printHtaccessCode($in{'pwf'});
		&printFooter;
		exit;
	}
}


sub removeUsername {
	$tempAction = "Removing username(s) ";
	while (($key,$val) = each %toRemove) { $tempAction .= "\"$key\","; }
	chop ($tempAction);
	$tempAction .= " from file \"$in{'pwf'}\".";
	unless (keys(%toRemove)) { &printResult($tempAction,'Failed','No username given.'); &listUsernames($in{'pwf'}); &printFooter; exit; }
	open (PWF,"./data/$in{'pwf'}.pwf");
	@oldPWF = (<PWF>);
	close (PWF_);
	foreach (@oldPWF) {
		($tempUsername,$d) = split (/:/,$_);
		if ($toRemove{$tempUsername}) { $removed{$tempUsername} = 1; }
		else { $newPWF .= $_; }
	}
	if ($in{'pwf'} eq "SPMadmin" && $newPWF eq "") { 
		&printResult($tempAction,'Failed','The username(s) were not removed as you need at least one username/password pair in the "SPMadmin" file (otherwise you will not be able to access this script).');
		&listUsernames($in{'pwf'});
	}
	else {
		open (PWF,">./data/$in{'pwf'}.pwf");
		print PWF $newPWF;
		close (PWF);
		$failure = 0;
		while (($key,$val) = each %toRemove) { unless ($removed{$key}) { $failure = 1; push (@notRemoved,$key); } }
		if ($failure) {
			foreach (@notRemoved) { $tempResult .= "\"$_\","; }
			chop ($tempResult);
			&printResult($tempAction,'Failed','The username(s) '.$tempResult.' was not removed.');
			&listUsernames($in{'pwf'});
		}
		else { &printResult($tempAction,'Success',''); }
	}
	&printFooter;
	exit;
}


sub addForm {
	$thisPWF = shift;
	if ($thisPWF eq "SPMadmin") {
		unless (-e "./data/SPMadmin.pwf") { $fileSelection = "<input type=hidden name=pwf value=\"$thisPWF\">$thisPWF"; }
	}
	unless ($fileSelection) {
		opendir (DIR,"$thisDir/data/");
		@pwfs = grep /\.pwf$/, readdir DIR;
		closedir (DIR);
		$fileSelection = 	"<select name=pwf>\n".
							"<option value=\"\">Please Select:</option>\n";
		foreach $file (@pwfs) {
			$file =~ s/\.pwf$//g;
			$fileSelection .= "<option";
			if ($file eq $in{'pwf'}) { $fileSelection .= " selected"; }
			$fileSelection .= ">$file</option>\n";
		}
		$fileSelection .= "</select>\n";
	}
	print 	"<p><form action=\"$ENV{'SCRIPT_NAME'}\" METHOD=GET><input type=hidden name=function value=\"Add\">\n".
			"<table cellpadding=0 cellspacing=0 bgcolor=#000000 border=0><tr><td>\n".
			"<table cellpadding=4 cellspacing=1 border=0>\n".
			"<tr><td bgcolor=#C0C0C0><b>Username:</b></td><td bgcolor=#FFFFFF><input type=text name=username size=30 value=\"$in{'username'}\"></td></tr>\n".
			"<tr><td bgcolor=#C0C0C0><b>Password:</b></td><td bgcolor=#FFFFFF><input type=text name=password size=30 value=\"$in{'password'}\"></td></tr>\n".
			"<tr><td bgcolor=#C0C0C0><b>File:</b></td><td bgcolor=#FFFFFF>$fileSelection</td></tr>\n".
			"<tr><td bgcolor=#C0C0C0>&nbsp;</td><td bgcolor=#FFFFFF><input type=submit value=\"Add Username\"></td></tr>\n".
			"</table>\n".
			"</td></tr></table></form></p><p>&nbsp;</p>\n";
}


sub listForm {
	opendir (DIR,"$thisDir/data/");
	@pwfs = grep /\.pwf$/, readdir DIR;
	closedir (DIR);
	$fileSelection = 	"<select name=pwf>\n".
						"<option value=\"\">Please Select:</option>\n";
	foreach $file (@pwfs) {
		$file =~ s/\.pwf$//g;
		$fileSelection .= "<option";
		if ($file eq $in{'pwf'}) { $fileSelection .= " selected"; }
		$fileSelection .= ">$file</option>\n";
	}
	$fileSelection .= "</select>\n";
	print 	"<p><form action=\"$ENV{'SCRIPT_NAME'}\" METHOD=GET><input type=hidden name=function value=\"List\">\n".
			"<table cellpadding=0 cellspacing=0 bgcolor=#000000 border=0><tr><td>\n".
			"<table cellpadding=4 cellspacing=1 border=0>\n".
			"<tr><td bgcolor=#C0C0C0><b>Filename:</b></td><td bgcolor=#FFFFFF>$fileSelection</td></tr>\n".
			"<tr><td bgcolor=#C0C0C0>&nbsp;</td><td bgcolor=#FFFFFF><input type=submit value=\"List Usernames\"></td></tr>\n".
			"</table>\n".
			"</td></tr></table></form></p><p>&nbsp;</p>\n";
}


sub fileForm {
	print 	"<p><form action=\"$ENV{'SCRIPT_NAME'}\" METHOD=GET><input type=hidden name=function value=\"New\">\n".
			"<table cellpadding=0 cellspacing=0 bgcolor=#000000 border=0><tr><td>\n".
			"<table cellpadding=4 cellspacing=1 border=0>\n".
			"<tr><td bgcolor=#C0C0C0><b>Filename:</b></td><td bgcolor=#FFFFFF><input type=text name=pwf size=30 value=\"$in{'pwf'}\"></td></tr>\n".
			"<tr><td bgcolor=#C0C0C0>&nbsp;</td><td bgcolor=#FFFFFF><input type=submit value=\"Create New Password File\"></td></tr>\n".
			"</table>\n".
			"</td></tr></table></form></p><p>&nbsp;</p>\n";
}


sub printResult {
	($action,$result,$reason) = @_;
	print 	"<p><table width=600 cellpadding=0 cellspacing=0 bgcolor=#000000 border=0><tr><td>\n".
			"<table width=600 cellpadding=4 cellspacing=1 border=0>\n".
			"<tr><td bgcolor=#C0C0C0><b>Action:</b></td><td bgcolor=#FFFFFF>$action</td></tr>\n".
			"<tr><td bgcolor=#C0C0C0><b>Result:</b></td><td bgcolor=#FFFFFF>$result</td></tr>\n";
	if ($reason) {
		print	"<tr><td bgcolor=#C0C0C0><b>Reason:</b></td><td bgcolor=#FFFFFF>$reason</td></tr>\n";
	}
	print	"</table>\n".
			"</td></tr></table></p><p>&nbsp;</p>\n";
}


sub printHtaccessCode {
	$thisPWF = shift;
	print 	"<p>Copy the text in the box below, place in a text file called \".htaccess\" and upload it to the directory you would like to password protect.<form><textarea cols=50 rows=10 wrap=off>".
			"AuthUserFile $thisDir/data/$thisPWF\.pwf\n".
			"AuthGroupFile /dev/null\n".
			"AuthName \"Private Area\"\n".
			"AuthType Basic\n\n".
			"<LIMIT GET POST PUT>\n".
			"require valid-user\n".
			"</LIMIT>\n".
			"</textarea></form></p>\n";
}


sub parseInput {
	@pairs = split(/&/, $ENV{'QUERY_STRING'});
	foreach $pair (@pairs) {
		($name, $value) = split(/=/, $pair);
		$value =~ tr/+/ /;
		$name =~ tr/+/ /;
		$value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
		$name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
		$value =~ s/\cM//g;
		$name =~ s/\cM//g;
		if ($name eq "remove") { $toRemove{$value} = 1; }
		else { $in{$name} = $value; }
	}
}


sub printHeader {
	print "Content-type: text/html\n\n";
	print <<"END";
<html>
<head>

<title>Simple Password Manager</title>

<style Type="Text/CSS">
<!--
	BODY,TD,P,LI,DT,DD	{font-family:Verdana;font-size:8pt}
	A					{font-family:Verdana;text-decoration: none;font-size:8pt}
	A:hover 			{text-decoration: underline;}
//-->
</style>

</head>
<body bgColor="#FFFFFF" Link="#0000FF" aLink="#FF0000" vLink="#0000FF" Text="#000000">
<p>
	<font size=2><b>Simple Password Manager</b></font>
	<br>
END
	print "<a href=\"$ENV{'SCRIPT_NAME'}\"><b>Main Page</b></a>\n</p>\n";
};


sub printFooter {
	print <<"END";
<p>&nbsp;</p>
<p>
	Simple Password Manager (SPM) v 1.0
	<br>
	&copy; 2001
	<a href="http://www.usefulscripts.com/">UsefulScripts.com</a>
	and iNet Publishing Ltd
	<br>
	No warranty expressed or implied.
</p>
END
};