#!/usr/bin/perl

###############################################################
# Link Admin v 1.0 Link Submission Script                     #
###############################################################
# submit.cgi v 1.0 RJ 13-Nov-2001                             #
# This contains the basic functions for processing a new link #
# submission.                                                 #
#                                                             #
# For further information please visit:                       #
# http://www.UsefulScripts.com/                               #
###############################################################
# Copyright  2001 iNet Publishing Ltd                        #
# Reproduction in part or in whole is strictly prohibited.    #
# This copyright header must not be removed.                  #
# No warranty expressed or implied.                           #
###############################################################

# Version 1.1 26-Jan-2004
# Added additional code for new version of Perl handling DBM files

use DB_File;

# Load in the configuration file. This is the only hard coded
# part of the script.
dbmopen (%CONF,"%%EDITCONF:workingDirectory::EDIT%%/config/core.Config",0766);
$DATA{'core.Submit.Version'} = "1.0";


###############################################################
#           DO NOT EDIT ANYTHING BENEATH THIS LINE!           #
###############################################################

# Load in the core library containing shared functions
require ($CONF{'scriptWorkingDir'} . "library/core.Library.pl");

# Load in the module for the specific type of data storage being used
require ($CONF{'scriptWorkingDir'} . "library/module." . $CONF{'dataStorage'} . ".pl");

&parseFormInput;

# Initiate the checking of the submission
$submissionError = 0;

# If the Black List is built-in
if ($CONF{'usBlackList'}) {
	require "./usBlackListCheck.pl";
	$usBlackListCheckResult = &usBlackListCheckBlock($in{'siteURL'},$in{'webmasterEmail'});
	if ($usBlackListCheckResult =~ m|^blocked|i) {
		print "Location: http://bl.usefulscripts.com/blocked/?reason=$usBlackListCheckResult\n\n";
		exit;
	}
}

# Set the IP address
$in{'webmasterIPaddress'} = $ENV{'REMOTE_ADDR'};

# Check that the IP address hasn;t already submitted today if the option is selected
if ($CONF{'multiIPBlocking'} && !$in{'friendSite'}) {
	if (&checkIPaddress) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"multiIPaddress"}."\n"; }
}

# Check all required fields are completed
@requiredFields = &getRequiredSubmitFields;
foreach (@requiredFields) {
	if (!$in{$_}) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{$_}."\n"; }
}

# Check that the email address provided appears valid
if ($in{'webmasterEmail'} && !&checkEmailSyntax($in{'webmasterEmail'})) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"invalidEmail"}."\n"; }

# Extract host for the submitted site
$siteDomain = &parseURL($in{'siteURL'});
# Check syntax of site URL
if ($in{'siteURL'} && !&checkURLSyntax($siteDomain)) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"invalidSiteURL"}."$siteDomain\n"; }

# Check syntax of recip URL (if recips are required)
if ($CONF{'recipLink'}) { 
	# Extract host for the reciprocal link
	$recipDomain = &parseURL($in{'recipURL'});
	if ($in{'recipURL'}) {
		if (!&checkURLSyntax($recipDomain)) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"invalidRecipURL"}."\n"; }
	}
}

# Check if the site URL is already listed if check duplicate URLs is set
if ($in{'siteURL'} && $CONF{'checkDupURLs'}) { 
	if (&checkDupURL) { $submissionError = 1; $errorResponse .= "<li>".$englishErrorMessages{"duplicateURL"}."\n"; }
}

# Check the submission against the blocked hosts
$siteDomain =~ s/^www\.//g;
$siteDomainWWW = "www." . $siteDomain;
open (BLOCKED,$CONF{'scriptWorkingDir'} . "data/data.Blocked.txt"); &rLock('BLOCKED');
@blockedHosts = (<BLOCKED>);
&unLock('BLOCKED'); close (BLOCKED);

$blocked = 0;

foreach (@blockedHosts) {
	chop($_);
	# We do it this way to avoid problems with TLDs such as com.au
	# This tests to see if the domain name is blocked
	if ($siteDomain =~ m|^$_$|i || $siteDomainWWW =~ m|^$_$|i || $siteDomain =~ m|^([a-z0-9-]*)\.$_$|i || $siteDomainWWW =~ m|^([a-z0-9-]*)\.$_$|i) {
		$blocked = 1; if ($CONF{'blockMessage'}) { $submissionError=1; $errorResponse .= "<li>".$englishErrorMessages{"blockedSite"}."\n"; }
	}
	# This tests to see if the blocked host contains any directory "/" and if so checks the site URL
	if ($_ =~ /\// && $in{'siteURL'} =~ m|$_|i) {
		$blocked = 1; if ($CONF{'blockMessage'}) { $submissionError=1; $errorResponse .= "<li>".$englishErrorMessages{"blockedSite"}."\n"; }
	}
	# This tests to see if the submitter's IP address is blocked
	if ($in{'webmasterIPaddress'} =~ m|^$_|i) {
		$blocked = 1; if ($CONF{'blockMessage'}) { $submissionError=1; $errorResponse .= "<li>".$englishErrorMessages{"blockedSite"}."\n"; }
	}
	# This tests to see if the submitter's email address is blocked
	if ($in{'webmasterEmail'} =~ m|^$_$|i || ($_ =~ m|^\@| && $in{'webmasterEmail'} =~ m|$_$|i)) {
		$blocked = 1; if ($CONF{'blockMessage'}) { $submissionError=1; $errorResponse .= "<li>".$englishErrorMessages{"blockedSite"}."\n"; }
	}
}

# If there's an error now, tell the webmaster before bothering to check for the recip link
if ($submissionError) { &printErrorMessage($englishErrorMessages{"submitErrorPrefix"},$errorResponse); exit; }

# If the submission is blocked and the Block Message is not set, then resirect them as if it were successful
if ($blocked && !$CONF{'blockMessage'}) { &sendEmail("EmailSubmit",""); print "Location: $CONF{'redirectURL'}\n\n"; exit; }

# If the link checking module is being used
if ($CONF{'linkChecking'}) { 
	# Load in the module for all the link checking, etc
	require ($CONF{'scriptWorkingDir'} . "library/module.LinkCheck.pl");

	# Check for the reciprocal link if it is required and the auto-check is set and the Link Checking module
	# is being used
	if ($in{'recipURL'} && $CONF{'recipLink'} && $CONF{'autoCheckRecip'} && $CONF{'linkChecking'}) {

		# We put a trailing slash ("/") at the end of the recip URL if it is just the domain name.
		if ($in{'recipURL'} !~ m|^http://([a-z0-9\._\-]*)/| && $in{'recipURL'} !~ m|^ftp://([a-z0-9\._\-]*)/|) { $in{'recipURL'} .= "/"; }

		if (!&checkRecipLink($in{'recipURL'})) { $submissionError=1; $errorResponse .= "<li>".$englishErrorMessages{"noRecipLink"}."\n"; }
		else { $in{'recipStatus'} = "found"; }
	}
	else { $in{'recipStatus'} = "not checked"; }


	# We put a trailing slash ("/") at the end of the URL if it is just the domain name.
	# This prevents the surfer being auto-redirected sometimes and speeds up the link.
	if ($in{'siteURL'} !~ m|^http://([a-z0-9\._\-]*)/| && $in{'siteURL'} !~ m|^ftp://([a-z0-9\._\-]*)/|) { $in{'siteURL'} .= "/"; }

	# If the Link Checking module is being used check that the response of the submitted site is 200 and that 
	# it is the proper URL
	$siteStatus = &checkLinkStatus($in{'siteURL'});
	if (!$siteStatus) { $siteStatus = "500 Unable To Connect"; }
	if ($siteStatus !~ /^200/ && $siteStatus !~ /^302 Found/) { $submissionError=1; $errorResponse .= "<li>".$englishErrorMessages{"invalidSiteResponse"}." \"$siteStatus\".\n"; }
}
elsif ($in{'recipURL'}) { $in{'recipStatus'} = "not checked"; }

if ($submissionError) { &printErrorMessage($englishErrorMessages{"submitErrorPrefix"},$errorResponse); exit; }

# Clean up the comments, so it is all on one line
$in{'webmasterComments'} =~ s/\cM//g; $in{'webmasterComments'} =~ s/\n/ /g;
$in{'webmasterComments'} =~ s/&lt;/</g; $in{'webmasterComments'} =~ s/&gt;/>/g;

# Default setting for all new submissions
$in{'siteStatus'} = "Pending";

# We're still here, so the submission must be okay to accept
if (&addSubmission) { 
	if ($CONF{'submitEmailNotify'}) { &sendEmail("EmailSubmit",""); }
	print "Location: $CONF{'redirectURL'}\n\n"; 
	exit;
}
else { &printErrorMessage($englishErrorMessages{"submitErrorPrefix"},"<li>" . $englishErrorMessages{"submitAppendFailure"}); exit; }

